import { SessionManager, KernelSpecManager } from '@jupyterlab/services';
import { UUID } from '@lumino/coreutils';
export class Kernel {
    _kernelSpecManager;
    _kernelManager;
    _sessionManager;
    _kernelName;
    _kernelConnection;
    _session;
    _path;
    _clientId;
    _id;
    _sessionId;
    _info;
    _connectionStatus;
    _readyResolve;
    _ready;
    constructor(props) {
        const { kernelManager, kernelName, kernelModel, serverSettings } = props;
        this._kernelSpecManager = new KernelSpecManager({ serverSettings });
        this._kernelManager = kernelManager;
        this._kernelName = kernelName;
        this.initReady = this.initReady.bind(this);
        this.initReady();
        this.requestJupyterKernel(kernelModel);
    }
    initReady() {
        this._ready = new Promise((resolve, reject) => {
            this._readyResolve = resolve;
        });
    }
    async requestJupyterKernel(kernelModel) {
        await this._kernelManager.ready;
        this._sessionManager = new SessionManager({
            kernelManager: this._kernelManager,
            serverSettings: this._kernelManager.serverSettings,
            standby: 'never',
        });
        await this._sessionManager.ready;
        if (kernelModel) {
            console.log('Reusing a pre-existing kernel model.');
            const model = this._sessionManager.running().next().value;
            this._session = this._sessionManager.connectTo({ model });
        }
        else {
            this._path = "kernel-" + UUID.uuid4();
            ;
            this._session = await this._sessionManager.startNew({
                name: this._path,
                path: this._path,
                type: this._kernelName,
                kernel: {
                    name: this._kernelName,
                },
            }, {
                kernelConnectionOptions: {
                    handleComms: true,
                }
            });
        }
        this._kernelConnection = this._session.kernel;
        const checkConnectionStatus = () => {
            if (this._connectionStatus === 'connected') {
                this._clientId = this._session.kernel.clientId;
                this._id = this._session.kernel.id;
                this._readyResolve();
            }
        };
        if (this._kernelConnection) {
            this._sessionId = this._session.id;
            this._connectionStatus = this._kernelConnection.connectionStatus;
            checkConnectionStatus();
            this._kernelConnection.connectionStatusChanged.connect((_, connectionStatus) => {
                //        this.initReady();
                this._connectionStatus = connectionStatus;
                checkConnectionStatus();
            });
            this._kernelConnection.info.then((info) => {
                this._info = info;
                console.log(`The default Kernel is ready`, this.toString());
            });
        }
    }
    get ready() {
        return this._ready;
    }
    get clientId() {
        return this._clientId;
    }
    get id() {
        return this._id;
    }
    get sessionId() {
        return this._sessionId;
    }
    get info() {
        return this._info;
    }
    get session() {
        return this._session;
    }
    get kernelManager() {
        return this._kernelManager;
    }
    get kerneSpeclManager() {
        return this._kernelSpecManager;
    }
    get sessionManager() {
        return this._sessionManager;
    }
    get path() {
        return this._path;
    }
    get connection() {
        return this._kernelConnection;
    }
    toString() {
        return `id:${this.id} client_id:${this.clientId} session_id:${this.sessionId} path:${this._path} kernelInfo:${this.info}`;
    }
    shutdown() {
        this._session.kernel?.shutdown();
        this.connection?.dispose();
    }
}
export default Kernel;
//# sourceMappingURL=Kernel.js.map